@@ -1,5 +1,4 @@
 import logging
-import os
 import tempfile
 from pathlib import Path
 
@@ -12,19 +11,8 @@ DEFAULT_LDFLAGS = ["-lrt", "-lm", "-lpthread", "-ldl"]
 
 
 class Runner(BaseRunner):
-
     log_prefix = "cpp-kernel"
     default_runtime_path = "/usr/bin/g++"
-    default_child_env = {
-        "TERM": "xterm",
-        "LANG": "C.UTF-8",
-        "SHELL": "/bin/ash",
-        "USER": "work",
-        "HOME": "/home/work",
-        "PATH": "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin",
-        "LD_LIBRARY_PATH": os.environ.get("LD_LIBRARY_PATH", ""),
-        "LD_PRELOAD": os.environ.get("LD_PRELOAD", ""),
-    }
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
@@ -50,7 +38,7 @@ class Runner(BaseRunner):
             cmd = [str(self.runtime_path), *map(str, ofiles_glob), *DEFAULT_CFLAGS, "-o", "./main"]
             return await self.run_subproc(cmd)
         else:
-            log.error('cannot find build script ("Makefile") ' 'or the main file ("main.cpp").')
+            log.error('cannot find build script ("Makefile") or the main file ("main.cpp").')
             return 127
 
     async def execute_heuristic(self) -> int:
